import React, { useEffect, useMemo, useRef } from "react";
import { register } from "@antv/x6-react-shape";
import { Graph, Node } from "@antv/x6";
import type { ColumnItem, TableItemType, ViewTable } from "@/type";
import { DATA_TYPE_OPTIONS } from "@/constants";
function TableNode({ node, graph }: { node: Node; graph: Graph }) {
  const { table, tableColumnList } = node.getData<TableItemType>();
  const containerRef = useRef<HTMLDivElement>(null);

  useEffect(() => {
    const container = containerRef.current;
    if (container?.clientHeight) {
      node.setSize(container.clientWidth, container.clientHeight);
    }
  }, [tableColumnList.length]);

  const FiledItem = ({ record }: { record: ColumnItem }) => {
    const type = DATA_TYPE_OPTIONS.find((item) => item.value === record.type);
    return (
      <div className="w-full flex py-4px px-8px">
        <span className="flex-1 truncate flex items-center justify-between">
          <span className="flex items-center">
            <span className=" w-6px h-6px rounded-full mr-4px bg-#5684bb inline-block "/>
            {record.schemaName}({record.cn_name})
          </span>
          <span>
            {record.isUnique ? (
              <span className="m-r-4px">
                <i className="iconfont icon-key-fill color-#efc553 m-r--4px" />
                <span className="text-12px color-#f15359">P</span>
                <span className="text-12px color-#8dafd1">F</span>
              </span>
            ) : (
              <span>
                <i className="iconfont icon-24_beizhu color-#efc553 m-r-4px text-12px" />
              </span>
            )}
            <span>{type?.label}</span>
          </span>
        </span>
      </div>
    );
  };

  return (
    <div
      ref={containerRef}
      className="w-full border border-1px border-solid border-#333 flex flex-col overflow-hidden rounded-8px"
    >
      <div
        className="w-full h-10px bg-#eee"
        style={{ background: table.style?.color || "#eee" }}
      ></div>
      <div
        className="
          header 
          border-b-solid 
          border-b-1px 
          border-b-#333 
          truncate 
          bg-#e4e4e7 
          py-4px 
          font-bold 
          text-14px 
          min-h-32px 
          px-8px
        "
      >
        {table.schemaName}({table.cn_name})
      </div>
      <div className="bg-#fafafa flex-1">
        <div className="field-info">
          {tableColumnList.map((item) => {
            return <FiledItem record={item} />;
          })}
        </div>
      </div>
    </div>
  );
}

register({
  shape: "table-node",
  component: TableNode,
  width: 300,
  height: 220,
  effect: ["data"],
});
